package seccomp

const (
	seccompOverwrite = "overwrite"
	seccompAppend    = "append"
	nothing          = "nothing"
	kill             = "kill"
	trap             = "trap"
	trace            = "trace"
	allow            = "allow"
	errno            = "errno"
)
